@@ -51,9 +51,9 @@
 //! the [NEWS.md](https://gitlab.com/ppentchev/expect-exit/-/blob/master/NEWS.md)
 //! file in the source distribution.
 
-#[doc(html_root_url = "https://docs.rs/expect-exit/0.4.0")]
-/**
- * Copyright (c) 2020, 2021  Peter Pentchev <roam@ringlet.net>
+#![doc(html_root_url = "https://docs.rs/expect-exit/0.4.2")]
+/*
+ * Copyright (c) 2020 - 2022  Peter Pentchev <roam@ringlet.net>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -77,12 +77,30 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  */
-use std::error;
-use std::fmt;
+// Turn most of clippy::restriction's lints on...
+#![warn(clippy::if_then_some_else_none)]
+#![warn(clippy::indexing_slicing)]
+#![warn(clippy::missing_const_for_fn)]
+#![warn(clippy::missing_docs_in_private_items)]
+#![warn(clippy::missing_inline_in_public_items)]
+#![warn(clippy::needless_pass_by_value)]
+#![warn(clippy::panic)]
+#![warn(clippy::print_stdout)]
+#![warn(clippy::print_stderr)]
+#![warn(clippy::single_char_lifetime_names)]
+#![warn(clippy::use_debug)]
+// ...except for these ones.
+#![allow(clippy::implicit_return)]
+
+use std::error::Error;
+use std::fmt::{Display, Formatter, Result as FmtResult};
 use std::panic;
 use std::process;
 
+/// Something to enqueue a [`Drop::drop()`] call to ensure that we unwind
+/// the stack before exiting the process.
 struct ExitHelper {
+    /// The code to exit the program with after the stack has been unwound.
     code: i32,
 }
 
@@ -93,33 +111,41 @@ impl Drop for ExitHelper {
 }
 
 /// Unwind the stack and end the process with the specified exit code.
+#[inline]
 pub fn exit_unwind(code: i32) -> ! {
     panic::resume_unwind(Box::new(ExitHelper { code }));
 }
 
+/// Output a message, then invoke the specified exit routine.
+#[allow(clippy::print_stderr)]
 fn _die(msg: &str, exit: fn(i32) -> !) -> ! {
     eprintln!("{}", msg);
     exit(1);
 }
 
-fn _die_perror(msg: &str, err: impl fmt::Display, exit: fn(i32) -> !) -> ! {
+/// Output a message containing an error description, then invoke the specified exit routine.
+#[allow(clippy::print_stderr)]
+fn _die_perror(msg: &str, err: impl Display, exit: fn(i32) -> !) -> ! {
     eprintln!("{}: {}", msg, err);
     exit(1);
 }
 
 /// Display the specified message, then unwind the stack and exit.
+#[inline]
 pub fn exit(msg: &str) -> ! {
     _die(msg, exit_unwind);
 }
 
 /// Display the specified message and append an appropriate
 /// description of the error, then unwind the stack and exit.
-pub fn exit_perror(msg: &str, err: impl fmt::Display) -> ! {
+#[inline]
+pub fn exit_perror(msg: &str, err: impl Display) -> ! {
     _die_perror(msg, err, exit_unwind);
 }
 
 /// Display the specified message, then exit without unwinding
 /// the stack.
+#[inline]
 pub fn die(msg: &str) -> ! {
     _die(msg, process::exit);
 }
@@ -127,23 +153,26 @@ pub fn die(msg: &str) -> ! {
 /// Display the specified message and append an appropriate
 /// description of the error, then exit without unwinding
 /// the stack.
-pub fn die_perror(msg: &str, err: impl fmt::Display) -> ! {
+#[inline]
+pub fn die_perror(msg: &str, err: impl Display) -> ! {
     _die_perror(msg, err, process::exit);
 }
 
 /// The error object returned by the [`ExpectedResult`] methods.
 #[derive(Debug)]
 pub struct ExpectationFailed {
+    /// The error message describing the failure that occurred.
     message: String,
 }
 
-impl fmt::Display for ExpectationFailed {
-    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+impl Display for ExpectationFailed {
+    #[inline]
+    fn fmt(&self, f: &mut Formatter<'_>) -> FmtResult {
         write!(f, "{}", self.message)
     }
 }
 
-impl error::Error for ExpectationFailed {}
+impl Error for ExpectationFailed {}
 
 /// Unwrap or exit with the specified message.
 pub trait Expected<T> {
@@ -240,25 +269,38 @@ pub trait ExpectedWithError<T>: Expected<T> {
 /// the specified error message.
 pub trait ExpectedResult<T> {
     /// Return a result object with a non-boxed `Error` object.
+    ///
+    /// # Errors
+    /// [`ExpectationFailed`] on error.
     fn expect_result_nb_(self, msg: &str) -> Result<T, ExpectationFailed>;
 
     /// Return a result object that may be tested using the `?` operator.
-    fn expect_result_(self, msg: &str) -> Result<T, Box<dyn error::Error>>;
+    ///
+    /// # Errors
+    /// [`ExpectationFailed`] on error.
+    fn expect_result_(self, msg: &str) -> Result<T, Box<dyn Error>>;
 
     /// Return a result object with a non-boxed `Error` object.
     /// Invoke the specified function to obtain the error message.
+    ///
+    /// # Errors
+    /// [`ExpectationFailed`] on error.
     fn expect_result_nb<F>(self, f: F) -> Result<T, ExpectationFailed>
     where
         F: FnOnce() -> String;
 
     /// Return a result object that may be tested using the `?` operator.
     /// Invoke the specified function to obtain the error message.
-    fn expect_result<F>(self, f: F) -> Result<T, Box<dyn error::Error>>
+    ///
+    /// # Errors
+    /// [`ExpectationFailed`] on error.
+    fn expect_result<F>(self, f: F) -> Result<T, Box<dyn Error>>
     where
         F: FnOnce() -> String;
 }
 
 impl<T> Expected<T> for Option<T> {
+    #[inline]
     fn or_exit_(self, msg: &str) -> T {
         match self {
             Some(value) => value,
@@ -266,6 +308,7 @@ impl<T> Expected<T> for Option<T> {
         }
     }
 
+    #[inline]
     fn or_exit<F>(self, f: F) -> T
     where
         F: FnOnce() -> String,
@@ -276,10 +319,12 @@ impl<T> Expected<T> for Option<T> {
         }
     }
 
+    #[inline]
     fn expect_or_exit_(self, msg: &str) -> T {
         self.or_exit_(msg)
     }
 
+    #[inline]
     fn expect_or_exit<F>(self, f: F) -> T
     where
         F: FnOnce() -> String,
@@ -287,6 +332,7 @@ impl<T> Expected<T> for Option<T> {
         self.or_exit(f)
     }
 
+    #[inline]
     fn or_die_(self, msg: &str) -> T {
         match self {
             Some(value) => value,
@@ -294,6 +340,7 @@ impl<T> Expected<T> for Option<T> {
         }
     }
 
+    #[inline]
     fn or_die<F>(self, f: F) -> T
     where
         F: FnOnce() -> String,
@@ -304,10 +351,12 @@ impl<T> Expected<T> for Option<T> {
         }
     }
 
+    #[inline]
     fn expect_or_die_(self, msg: &str) -> T {
         self.or_die_(msg)
     }
 
+    #[inline]
     fn expect_or_die<F>(self, f: F) -> T
     where
         F: FnOnce() -> String,
@@ -318,8 +367,9 @@ impl<T> Expected<T> for Option<T> {
 
 impl<T, E> Expected<T> for Result<T, E>
 where
-    E: fmt::Display,
+    E: Display,
 {
+    #[inline]
     fn or_exit_(self, msg: &str) -> T {
         match self {
             Ok(value) => value,
@@ -327,6 +377,7 @@ where
         }
     }
 
+    #[inline]
     fn or_exit<F>(self, f: F) -> T
     where
         F: FnOnce() -> String,
@@ -337,10 +388,12 @@ where
         }
     }
 
+    #[inline]
     fn expect_or_exit_(self, msg: &str) -> T {
         self.or_exit_(msg)
     }
 
+    #[inline]
     fn expect_or_exit<F>(self, f: F) -> T
     where
         F: FnOnce() -> String,
@@ -348,6 +401,7 @@ where
         self.or_exit(f)
     }
 
+    #[inline]
     fn or_die_(self, msg: &str) -> T {
         match self {
             Ok(value) => value,
@@ -355,6 +409,7 @@ where
         }
     }
 
+    #[inline]
     fn or_die<F>(self, f: F) -> T
     where
         F: FnOnce() -> String,
@@ -365,10 +420,12 @@ where
         }
     }
 
+    #[inline]
     fn expect_or_die_(self, msg: &str) -> T {
         self.or_die_(msg)
     }
 
+    #[inline]
     fn expect_or_die<F>(self, f: F) -> T
     where
         F: FnOnce() -> String,
@@ -379,8 +436,9 @@ where
 
 impl<T, E> ExpectedWithError<T> for Result<T, E>
 where
-    E: fmt::Display,
+    E: Display,
 {
+    #[inline]
     fn or_exit_e_(self, msg: &str) -> T {
         match self {
             Err(err) => exit_perror(msg, err),
@@ -388,10 +446,12 @@ where
         }
     }
 
+    #[inline]
     fn expect_or_exit_perror_(self, msg: &str) -> T {
         self.or_exit_e_(msg)
     }
 
+    #[inline]
     fn or_exit_e<F>(self, f: F) -> T
     where
         F: FnOnce() -> String,
@@ -402,6 +462,7 @@ where
         }
     }
 
+    #[inline]
     fn expect_or_exit_perror<F>(self, f: F) -> T
     where
         F: FnOnce() -> String,
@@ -409,6 +470,7 @@ where
         self.or_exit_e(f)
     }
 
+    #[inline]
     fn or_die_e_(self, msg: &str) -> T {
         match self {
             Err(err) => die_perror(msg, err),
@@ -416,6 +478,7 @@ where
         }
     }
 
+    #[inline]
     fn or_die_e<F>(self, f: F) -> T
     where
         F: FnOnce() -> String,
@@ -426,10 +489,12 @@ where
         }
     }
 
+    #[inline]
     fn expect_or_die_perror_(self, msg: &str) -> T {
         self.or_die_e_(msg)
     }
 
+    #[inline]
     fn expect_or_die_perror<F>(self, f: F) -> T
     where
         F: FnOnce() -> String,
@@ -439,87 +504,78 @@ where
 }
 
 impl<T> ExpectedResult<T> for Option<T> {
+    #[inline]
     fn expect_result_nb_(self, msg: &str) -> Result<T, ExpectationFailed> {
-        match self {
-            None => Err(ExpectationFailed {
-                message: msg.to_owned(),
-            }),
-            Some(value) => Ok(value),
-        }
+        self.ok_or_else(|| ExpectationFailed {
+            message: msg.to_owned(),
+        })
     }
 
-    fn expect_result_(self, msg: &str) -> Result<T, Box<dyn error::Error>> {
-        match self {
-            None => Err(Box::new(ExpectationFailed {
+    #[inline]
+    fn expect_result_(self, msg: &str) -> Result<T, Box<dyn Error>> {
+        self.ok_or_else(|| -> Box<dyn Error> {
+            Box::new(ExpectationFailed {
                 message: msg.to_owned(),
-            })),
-            Some(value) => Ok(value),
-        }
+            })
+        })
     }
+
+    #[inline]
     fn expect_result_nb<F>(self, f: F) -> Result<T, ExpectationFailed>
     where
         F: FnOnce() -> String,
     {
-        match self {
-            None => Err(ExpectationFailed { message: f() }),
-            Some(value) => Ok(value),
-        }
+        self.ok_or_else(|| ExpectationFailed { message: f() })
     }
 
-    fn expect_result<F>(self, f: F) -> Result<T, Box<dyn error::Error>>
+    #[inline]
+    fn expect_result<F>(self, f: F) -> Result<T, Box<dyn Error>>
     where
         F: FnOnce() -> String,
     {
-        match self {
-            None => Err(Box::new(ExpectationFailed { message: f() })),
-            Some(value) => Ok(value),
-        }
+        self.ok_or_else(|| -> Box<dyn Error> { Box::new(ExpectationFailed { message: f() }) })
     }
 }
 
 impl<T, E> ExpectedResult<T> for Result<T, E>
 where
-    E: fmt::Display,
+    E: Display,
 {
+    #[inline]
     fn expect_result_nb_(self, msg: &str) -> Result<T, ExpectationFailed> {
-        match self {
-            Ok(value) => Ok(value),
-            Err(err) => Err(ExpectationFailed {
-                message: format!("{}: {}", msg, err),
-            }),
-        }
+        self.map_err(|err| ExpectationFailed {
+            message: format!("{}: {}", msg, err),
+        })
     }
 
-    fn expect_result_(self, msg: &str) -> Result<T, Box<dyn error::Error>> {
-        match self {
-            Ok(value) => Ok(value),
-            Err(err) => Err(Box::new(ExpectationFailed {
+    #[inline]
+    fn expect_result_(self, msg: &str) -> Result<T, Box<dyn Error>> {
+        self.map_err(|err| -> Box<dyn Error> {
+            Box::new(ExpectationFailed {
                 message: format!("{}: {}", msg, err),
-            })),
-        }
+            })
+        })
     }
 
+    #[inline]
     fn expect_result_nb<F>(self, f: F) -> Result<T, ExpectationFailed>
     where
         F: FnOnce() -> String,
     {
-        match self {
-            Ok(value) => Ok(value),
-            Err(err) => Err(ExpectationFailed {
-                message: format!("{}: {}", f(), err),
-            }),
-        }
+        self.map_err(|err| ExpectationFailed {
+            message: format!("{}: {}", f(), err),
+        })
     }
 
-    fn expect_result<F>(self, f: F) -> Result<T, Box<dyn error::Error>>
+    #[inline]
+    fn expect_result<F>(self, f: F) -> Result<T, Box<dyn Error>>
     where
         F: FnOnce() -> String,
     {
-        match self {
-            Ok(value) => Ok(value),
-            Err(err) => Err(Box::new(ExpectationFailed {
+        self.map_err(|err| -> Box<dyn Error> {
+            Box::new(ExpectationFailed {
                 message: format!("{}: {}", f(), err),
-            })),
-        }
+            })
+        })
     }
 }
